#include <sys/param.h>
#include <sys/module.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/malloc.h>

static void runTest(void);
MALLOC_DECLARE(M_FOO);
MALLOC_DEFINE(M_FOO, "mfoo", "foo mem");

static int event_handler(struct module *module, int event, void *arg) {
    int e = 0;

    switch (event) {
    case MOD_LOAD:
        runTest();
        break;
    case MOD_UNLOAD:
        printf("Done\n");
        break;
    default:
        e = EOPNOTSUPP;
        break;
    }

    return(e);
}

static void runTest() {

        const int pageSize = 4096;
        unsigned long size;
        void *ptr = NULL;
        unsigned long long t1, t2;
        int i = 0;
        unsigned long long avg_malloc = 0, avg_free = 0;
        int probes = 40;
        size = 2 * 1024 * 1024;

        for (i = 0; i < probes; i++) {

                t1 = rdtsc();
                ptr = contigmalloc(size, M_FOO, 0, 0, ~0,
                                   pageSize, 0);
                t2 = rdtsc();

                avg_malloc += t2 - t1;

                printf("contigmalloc cycles: %llu\n", t2-t1);

                if (ptr == NULL)
                        return;

                t1 = rdtsc();
                contigfree(ptr, size, M_FOO);
                t2 = rdtsc();
                avg_free += t2 - t1;
                printf("contigfree cycles: %llu\n\n", t2-t1);
                uprintf("Loop complete\n");
        }

        printf("contigmalloc %llu contigfree %llu\n", avg_malloc / probes, avg_free / probes);
}

static moduledata_t test_conf = {
    "test",
     event_handler,
     NULL
};

DECLARE_MODULE(test, test_conf, SI_SUB_DRIVERS, SI_ORDER_MIDDLE);